# Import functions
. "$PSScriptRoot\functions\console.ps1"
. "$PSScriptRoot\functions\header.ps1"

# Import CSharp
Add-Type -Path "$PSScriptRoot\dependencies\JsonSchemaValidator.dll"


# Initialize
Write-Header

Get-Info

Write-Separator


# get all json files
$jsonFiles = Get-ChildItem -Recurse -Filter "*.json" -Exclude "*.schema.*" | select -ExpandProperty FullName
$jsonSchemaFiles = Get-ChildItem -Recurse -Filter "*.schema.json" | select -ExpandProperty FullName

# find schema that matches location and name of file
$errors = $False

$jsonFiles | Foreach-Object {
    $json = $_
    $jsonWithoutSchema = $json -replace "schema.", ""
    $schema = $jsonSchemaFiles | Where-Object {$($_ -replace "schema.", "") -eq $jsonWithoutSchema}
    
    if($schema) {
        $jsonContent = Get-Content $json | ConvertFrom-Json  -Depth 100 | ConvertTo-Json  -Depth 100
        $schemaContent = Get-Content $schema | ConvertFrom-Json  -Depth 100 | ConvertTo-Json  -Depth 100
        $result = $False
        Write-Separator

        Write-Host "Processing File: $(Remove-Current-Location -location $json)" 
        Write-Host "         Schema: $(Remove-Current-Location -location $schema)" 
        $result = [JsonSchemaValidator.JsonValidator]::Validate("$jsonContent", "$schemaContent")

        Write-Host "          Valid: $($result.Valid)" 
       
        if(!$result.Valid) {
            Write-Host "          Error: $($result.Errors | Select-Object * -ExcludeProperty schema | ConvertTo-Json -Depth 100)" 
            $errors = $True
        }
    }
}

if($errors) {
    Write-OutputCenter("{ There are schema validations that failed, failing the scan. }")
}

Write-Separator

Write-OutputCenter("And as always, have a nice day!")

Write-Separator
